Package: vip
Type: Package
Title: Variable Importance Plots
Version: 0.4.6
Authors@R: c(
  person(c("Brandon", "M."), family = "Greenwell", 
         email = "greenwell.brandon@gmail.com",
         role = c("aut", "cre"), 
         comment = c(ORCID = "0000-0002-8120-0084")),
  person("Brad", "Boehmke",
         email = "bradleyboehmke@gmail.com",
         role = c("aut"),
         comment = c(ORCID = "0000-0002-3611-8516"))
  )
Description: A general framework for constructing variable importance plots from 
  various types of machine learning models in R. Aside from some standard model-
  specific variable importance measures, this package also provides model-
  agnostic approaches that can be applied to any supervised learning algorithm.
  These include 1) an efficient permutation-based variable importance measure, 
  2) variable importance based on Shapley values (Strumbelj and Kononenko, 
  2014) <doi:10.1007/s10115-013-0679-x>, and 3) the variance-based 
  approach described in Greenwell et al. (2018) <doi:10.48550/arXiv.1805.04755>. A 
  variance-based method for quantifying the relative strength of interaction 
  effects is also included (see the previous reference for details).
License: GPL (>= 2)
URL: https://github.com/koalaverse/vip/,
        https://koalaverse.github.io/vip/
BugReports: https://github.com/koalaverse/vip/issues
Encoding: UTF-8
VignetteBuilder: knitr
Depends: R (>= 4.1.0)
Imports: foreach, ggplot2 (>= 0.9.0), stats, tibble, utils, yardstick
Suggests: bookdown, DT, covr, doParallel, dplyr, fastshap (>= 0.1.0),
        knitr, lattice, mlbench, modeldata, NeuralNetTools, pdp,
        rmarkdown, tinytest (>= 1.4.3), varImp
Enhances: C50, caret, Cubist, earth, gbm, glmnet, h2o, lightgbm,
        mixOmics, mlr, mlr3, neuralnet, nnet, parsnip (>= 0.1.7),
        party, partykit, pls, randomForest, ranger, rpart, RSNNS,
        sparklyr (>= 0.8.0), tidymodels, workflows (>= 0.2.3), xgboost
        (>= 3.2.0)
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-22 19:28:15 UTC; greenwbm
Author: Brandon M. Greenwell [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8120-0084>),
  Brad Boehmke [aut] (ORCID: <https://orcid.org/0000-0002-3611-8516>)
Maintainer: Brandon M. Greenwell <greenwell.brandon@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-23 07:20:02 UTC
Built: R 4.6.0; ; 2026-04-29 04:18:17 UTC; unix
