# Generated by roxygen2: do not edit by hand

S3method(vi,default)
S3method(vi_firm,default)
S3method(vi_model,C5.0)
S3method(vi_model,H2OBinomialModel)
S3method(vi_model,H2OMultinomialModel)
S3method(vi_model,H2ORegressionModel)
S3method(vi_model,Learner)
S3method(vi_model,RandomForest)
S3method(vi_model,WrappedModel)
S3method(vi_model,cforest)
S3method(vi_model,constparty)
S3method(vi_model,cubist)
S3method(vi_model,cv.glmnet)
S3method(vi_model,default)
S3method(vi_model,earth)
S3method(vi_model,gbm)
S3method(vi_model,glmnet)
S3method(vi_model,lgb.Booster)
S3method(vi_model,lm)
S3method(vi_model,mixo_pls)
S3method(vi_model,mixo_spls)
S3method(vi_model,ml_model_decision_tree_classification)
S3method(vi_model,ml_model_decision_tree_regression)
S3method(vi_model,ml_model_gbt_classification)
S3method(vi_model,ml_model_gbt_regression)
S3method(vi_model,ml_model_generalized_linear_regression)
S3method(vi_model,ml_model_linear_regression)
S3method(vi_model,ml_model_random_forest_classification)
S3method(vi_model,ml_model_random_forest_regression)
S3method(vi_model,mlp)
S3method(vi_model,model_fit)
S3method(vi_model,mvr)
S3method(vi_model,nn)
S3method(vi_model,nnet)
S3method(vi_model,randomForest)
S3method(vi_model,ranger)
S3method(vi_model,rpart)
S3method(vi_model,train)
S3method(vi_model,workflow)
S3method(vi_model,xgb.Booster)
S3method(vi_permute,default)
S3method(vi_shap,default)
S3method(vip,Learner)
S3method(vip,WrappedModel)
S3method(vip,default)
S3method(vip,model_fit)
S3method(vip,workflow)
export(gen_friedman)
export(list_metrics)
export(vi)
export(vi_firm)
export(vi_model)
export(vi_permute)
export(vi_shap)
export(vip)
importFrom(foreach,"%do%")
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(stats,reorder)
