% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_feature_names.R
\name{get_feature_names}
\alias{get_feature_names}
\title{Extract feature names}
\usage{
get_feature_names(object, ...)
}
\arguments{
\item{object}{An appropriate fitted model object.}

\item{...}{Additional optional arguments.}
}
\description{
Extract predictor names from a fitted model.
}
\keyword{internal}
